require 'common.util'

--[[
idunno about holding the last pos here
esp when the calling code needs it too
that means either callback (since assigning the last pos/buttons goes last)
or a separate after-the-fact update (what we're doing now)
or a lua-based mouse handler for last state
or exposure for the C++-based mouse last state (yay)
--]]

Ortho = class()
function Ortho:init(args)
	self.pos = args.pos or {0,0} 
	self.scale = args.scale or 1
	self.mousePos = {0,0}
	self.mouseButtons = 0
	self.mouseLastPos = {0,0}
	self.mouseLastButtons = 0
end

function Ortho:update()
	local aspectRatio
	do
		local vp = gl.glGet(gl.GL_VIEWPORT)
		aspectRatio = vp[3] / vp[4]
	end
	gl.glMatrixMode(gl.GL_PROJECTION)
	gl.glLoadIdentity()
	gl.glOrtho( -aspectRatio/2, aspectRatio/2, -1/2, 1/2, -1,1)
	gl.glMatrixMode(gl.GL_MODELVIEW)
	gl.glLoadIdentity()
	do
		local es = math.exp(self.scale)
		gl.glScale(es,es,1)
	end
	gl.glTranslate(self.pos[1], self.pos[2], 0)
	self.mousePos[1], self.mousePos[2], self.mouseButtons, self.mouseDZ = glush.mouse()

	local dx = self.mousePos[1] - self.mouseLastPos[1]
	local dy = self.mousePos[2] - self.mouseLastPos[2]

	-- handle input here ...

	if self.mouseButtons ~= 0 then
		if self.mouseButtons == 1 then
			self.pos[1] = self.pos[1] + dx * aspectRatio * math.exp(-self.scale)
			self.pos[2] = self.pos[2] + dy * math.exp(-self.scale)
		end
		if self.mouseButtons == 2 then
			self.scale = self.scale + 5 * dy
		end
	end
	-- TODO: offset to mouse pos (in self.pos coords), then zoom, then ofs-back
	if self.mouseDZ ~= 0 then
		self.pos[1] = self.pos[1] + (.5 - self.mousePos[1]) * aspectRatio * math.exp(-self.scale)
		self.pos[2] = self.pos[2] + (.5 - self.mousePos[2]) * math.exp(-self.scale)

		local ddz = self.mouseDZ * .02
		self.scale = self.scale + ddz

		self.pos[1] = self.pos[1] - (.5 - self.mousePos[1]) * aspectRatio * math.exp(-self.scale)
		self.pos[2] = self.pos[2] - (.5 - self.mousePos[2]) * math.exp(-self.scale)
	end
end


-- last store for next iteration
function Ortho:postUpdate()
	self.mouseLastPos[1] = self.mousePos[1]
	self.mouseLastPos[2] = self.mousePos[2]
	self.mouseLastButtons = self.mouseButtons
end

