--[[

    File: automata.lua 

    Copyright (C) 2000-2009 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

--]]

require 'common.glutil'
require 'common.pingpong'
require 'common.shader'

glush.gui.checkbox('reset', false)
glush.gui.slider('delay', 0, .1, 10)

-- slow copy method
fieldDim = 256

--[[
dead ->
	3 -> live
	otherwise -> dead
live ->
	2 or 3 -> live
	otherwise dead
--]]

-- this shader was made to be run 20x over
-- i'm adding sources in here too.  less passes.
stepShader = KernelShader.new{
	texs={'fieldTex'};
	uniforms={delta='vec2';};
	code=[[
void main() {
	vec4 ll = texture2D(fieldTex, pos + vec2(-delta.x, -delta.y));
	vec4 lm = texture2D(fieldTex, pos + vec2(0., -delta.y));
	vec4 lr = texture2D(fieldTex, pos + vec2(delta.x, -delta.y));
	vec4 ml = texture2D(fieldTex, pos + vec2(-delta.x, 0.));
	vec4 mm = texture2D(fieldTex, pos + vec2(0., 0.));
	vec4 mr = texture2D(fieldTex, pos + vec2(delta.x, 0.));
	vec4 rl = texture2D(fieldTex, pos + vec2(-delta.x, delta.y));
	vec4 rm = texture2D(fieldTex, pos + vec2(0., delta.y));
	vec4 rr = texture2D(fieldTex, pos + vec2(delta.x, delta.y));

	float neighbors = ll.x + lm.x + lr.x + ml.x + mr.x + rl.x + rm.x + rr.x;
	
	/**/
	float v = 0.;
	if (mm.x == 0. && neighbors == 3.) v = 1.;
	if (mm.x == 1. && (neighbors == 2. || neighbors == 3.)) v = 1.;
	/**/
	
	/** /
	float v = 0.;
	if (neighbors >= 2. - mm.x && neighbors <= 3.) v = 1.;
	/**/
	
	/** /
	float center = 2. + .5 * mm.x;	// we need everything within .7 <- [.5,1] of our epsilon to be true
	float v = neighbors - center;
	v = .75 - v*v;
	v *= 1000.;
	v = clamp(v, 0., 1.);
	/**/

	gl_FragColor = vec4(v);
}
]];
}

fieldDimViewport = {0, 0, fieldDim, fieldDim}
fieldDelta = {1/fieldDim, 1/fieldDim}		-- hardcode in shaders?

-- rgba: u,v, density
fieldTexs = PingPong.new{
	width=fieldDim;
	height=fieldDim;
	format=gl.GL_RGB;
	minFilter=gl.GL_NEAREST;
	magFilter=gl.GL_NEAREST;
}

function resetAll()
	fieldTexs:clearAll()
end
resetAll()

-- notice - this mucks up args

function iterate()
	-- shift
	fieldTexs:swap()
	fieldTexs:draw{
		texs={fieldTexs:prev()};
		shader=stepShader;
		uniforms={delta=fieldDelta;};
	}
end

function addMouseSource(pos)
	fieldTexs:swap()
	fieldTexs:draw{
		texs={fieldTexs:prev()};
		callback=function()
			-- screen overlay
			gl.glEnable(gl.GL_TEXTURE_2D)
			gl.glColor(1,1,1,1)
			drawScreenQuad()
			gl.glDisable(gl.GL_TEXTURE_2D)
			-- mouse influence
			gl.glColor(1,1,1,1)
			gl.glEnable(gl.GL_POINT_SMOOTH)
			gl.glBegin(gl.GL_POINTS)
			gl.glVertex(pos)
			gl.glEnd()
		end;
	}
end

gl.glDisable(gl.GL_DEPTH_TEST)

--hack
firstReset=2

local lasttime = 0
local totaldelta = 0
local lastFieldMousePos = {0,0}
function glush.display()
	glreport('begin display')
	
	if reset or firstReset>0 then
		resetAll()
		reset = false
		firstReset = firstReset-1
	end

	local viewport = gl.glGet(gl.GL_VIEWPORT)
	local aspectRatio = viewport[3] / viewport[4]
	
	-- set up for viewport fbo once
	-- assert that all your fbo targets are the same size
	gl.glViewport(0,0,fieldDim,fieldDim)
	gl.glMatrixMode(gl.GL_PROJECTION)
	gl.glLoadIdentity()
	gl.glOrtho(0,1,0,1,-1,1)
	gl.glMatrixMode(gl.GL_MODELVIEW)
	gl.glLoadIdentity()

	local mouseX,mouseY,mouseButton = glush.mouse()
	local fieldMousePos = { mouseX*aspectRatio - (aspectRatio-1)/2, mouseY }
	local fieldMouseVel = {
		(fieldMousePos[1] - lastFieldMousePos[1]),
		(fieldMousePos[2] - lastFieldMousePos[2])
	}
	if mouseButton ~= 0 then
		addMouseSource(fieldMousePos)
	end
	
	lastFieldMousePos[1] = fieldMousePos[1]
	lastFieldMousePos[2] = fieldMousePos[2]

	local doupdate = false
	local thistime = glush.timer()
	deltatime = thistime - lasttime
	totaldelta = totaldelta + deltatime
	if totaldelta >= delay then
		totaldelta = 0
		doupdate = true
	end
	lasttime = thistime

	if doupdate then
		iterate()
	end
	

	-- set up for rendering
	gl.glViewport(unpack(viewport))
	gl.glMatrixMode(gl.GL_PROJECTION)
	gl.glLoadIdentity()
	gl.glOrtho(-(aspectRatio-1)/2, 1+(aspectRatio-1)/2, 0, 1, -1, 1)
	gl.glMatrixMode(gl.GL_MODELVIEW)
	gl.glLoadIdentity()

	gl.glColor(1,1,1)
	gl.glEnable(gl.GL_TEXTURE_2D)
	fieldTexs:cur():bind()
	drawScreenQuad()
	fieldTexs:cur():unbind()
	gl.glDisable(gl.GL_TEXTURE_2D)
	
	glreport('end display')
end
